"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeorm_1 = require("typeorm");
const DriverUtils_1 = require("typeorm/driver/DriverUtils");
const filter_query_builder_1 = require("./filter-query.builder");
const query_builder_abstract_1 = require("./query-builder.abstract");
/**
 * @internal
 *
 * Class that will convert a Query into a `typeorm` Query Builder.
 */
class RelationQueryBuilder extends query_builder_abstract_1.AbstractQueryBuilder {
    constructor(repo, relation) {
        super(repo);
        this.repo = repo;
        this.relation = relation;
        this.relationRepo = this.repo.manager.getRepository(this.relationMeta.from);
        this.filterQueryBuilder = new filter_query_builder_1.FilterQueryBuilder(this.relationRepo);
    }
    select(entity, query) {
        const entityArr = Array.isArray(entity) ? entity : [entity];
        const relationBuilder = this.createRelationQueryBuilder(entityArr);
        return this.applyRelationFilter(relationBuilder, entityArr, query);
    }
    applyRelationFilter(baseBuilder, entityArr, query) {
        const { entityIds, fromPrimaryKeys, entityIdSelects } = this.relationMeta;
        const parameters = {};
        const fromPrimaryKeysIn = fromPrimaryKeys.length === 1
            ? fromPrimaryKeys[0].selectPath
            : `(${fromPrimaryKeys.map(pk => pk.selectPath).join(',')})`;
        const subQueries = entityArr.reduce((sqls, e, index) => {
            const { subQuery, params } = this.createRelationSubQuery(e, index);
            // only apply filter and paging
            let filteredSubQuery = this.filterQueryBuilder.applyFilter(subQuery, query.filter, subQuery.alias);
            filteredSubQuery = this.filterQueryBuilder.applyPaging(filteredSubQuery, query.paging);
            Object.assign(parameters, params, filteredSubQuery.getParameters());
            return [...sqls, `(${fromPrimaryKeysIn} IN (${filteredSubQuery.getQuery()}))`];
        }, []);
        const oredBuilder = baseBuilder.andWhere(`(${subQueries.join(' OR ')})`, parameters);
        // force sorting by entity pks first so sub results are sorted properly
        const sortedBuilder = entityIds
            .reduce((builder, pk) => {
            return builder.addOrderBy(pk, 'ASC');
        }, oredBuilder)
            .addSelect(entityIdSelects);
        return this.filterQueryBuilder.applySorting(sortedBuilder, query.sorting, sortedBuilder.alias);
    }
    createRelationSubQuery(e, entityIndex) {
        const meta = this.relationMeta;
        const baseBuilder = this.createRelationQueryBuilder().select(meta.fromPrimaryKeys.map(pk => pk.selectPath));
        const { sql, params } = meta.whereCondition(e, entityIndex);
        return { subQuery: baseBuilder.where(sql), params };
    }
    createRelationQueryBuilder(entities) {
        const meta = this.relationMeta;
        const queryBuilder = this.relationRepo.createQueryBuilder(meta.fromAlias);
        const joinedBuilder = meta.joins.reduce((qb, join) => {
            const conditions = join.conditions.map(({ leftHand, rightHand }) => `${leftHand} = ${rightHand}`);
            return qb.innerJoin(join.target, join.alias, conditions.join(' AND '));
        }, queryBuilder);
        if (!entities || !entities.length) {
            return joinedBuilder;
        }
        return joinedBuilder.andWhere(new typeorm_1.Brackets(andBuilder => {
            return entities.reduce((qb, entity, entityIndex) => {
                return qb.orWhere(new typeorm_1.Brackets(bqb => {
                    const where = meta.whereCondition(entity, entityIndex);
                    bqb.orWhere(where.sql, where.params);
                }));
            }, andBuilder);
        }));
    }
    get relationMeta() {
        if (this.relationMetadata) {
            return this.relationMetadata;
        }
        const relation = this.repo.metadata.relations.find(r => r.propertyName === this.relation);
        if (!relation) {
            throw new Error(`Unable to find entity for relation '${this.relation}'`);
        }
        else if (relation.isManyToOne || relation.isOneToOneOwner) {
            this.relationMetadata = this.getManyToOneOrOneToOneOwnerMeta(relation);
        }
        else if (relation.isOneToMany || relation.isOneToOneNotOwner) {
            this.relationMetadata = this.getOneToManyOrOneToOneNotOwnerMeta(relation);
        }
        else if (relation.isManyToManyOwner) {
            this.relationMetadata = this.getManyToManyOwnerMeta(relation);
        }
        else {
            // many-to-many non owner
            this.relationMetadata = this.getManyToManyNotOwnerMetadata(relation);
        }
        return this.relationMetadata;
    }
    getManyToOneOrOneToOneOwnerMeta(relation) {
        const aliasName = relation.entityMetadata.name;
        const { primaryColumns } = relation.entityMetadata;
        const fromPrimaryKeys = relation.inverseRelation.entityMetadata.primaryColumns.map(pk => ({
            selectPath: `${relation.propertyName}.${pk.propertyName}`,
            databasePath: pk.databasePath,
            propertyName: pk.propertyName,
        }));
        const { joinColumns } = relation;
        const joins = [
            {
                target: relation.entityMetadata.target,
                alias: aliasName,
                conditions: joinColumns.map(joinColumn => ({
                    leftHand: `${aliasName}.${joinColumn.propertyName}`,
                    rightHand: `${relation.propertyName}.${joinColumn.referencedColumn.propertyName}`,
                })),
            },
        ];
        const entityIds = primaryColumns.map(column => `${aliasName}.${column.propertyPath}`);
        const entityIdSelects = primaryColumns.map(column => `${aliasName}.${column.propertyPath} AS ${this.createEntityIdSelect(column.propertyPath)}`);
        return {
            from: relation.type,
            fromAlias: relation.propertyName,
            fromPrimaryKeys,
            entityIds,
            entityIdSelects,
            joins,
            whereCondition(entity, entityIndex) {
                const params = {};
                const sql = primaryColumns
                    .map((column, columnIndex) => {
                    const paramName = `${aliasName}_entity_${entityIndex}_${columnIndex}`;
                    params[paramName] = column.getEntityValue(entity);
                    return `${aliasName}.${column.propertyPath} = :${paramName}`;
                })
                    .join(' AND ');
                return { sql, params };
            },
        };
    }
    getOneToManyOrOneToOneNotOwnerMeta(relation) {
        const aliasName = relation.propertyName;
        const columns = relation.inverseRelation.joinColumns;
        const fromPrimaryKeys = relation.inverseRelation.entityMetadata.primaryColumns.map(pk => ({
            selectPath: `${aliasName}.${pk.propertyName}`,
            databasePath: pk.databasePath,
            propertyName: pk.propertyName,
        }));
        const entityIds = columns.map(joinColumn => `${aliasName}.${joinColumn.propertyPath}`);
        const entityIdSelects = columns.map(joinColumn => `${aliasName}.${joinColumn.propertyPath} AS ${this.createEntityIdSelect(joinColumn.referencedColumn.propertyPath)}`);
        return {
            from: relation.inverseRelation.entityMetadata.target,
            fromAlias: aliasName,
            fromPrimaryKeys,
            entityIds,
            entityIdSelects,
            joins: [],
            whereCondition(entity, entityIndex) {
                const params = {};
                const sql = columns
                    .map((col, colIndex) => {
                    const paramName = `${aliasName}_entity_${entityIndex}_${colIndex}`;
                    params[paramName] = col.referencedColumn.getEntityValue(entity);
                    return `${aliasName}.${col.propertyPath} = :${paramName}`;
                })
                    .join(' AND ');
                return { sql, params };
            },
        };
    }
    getManyToManyOwnerMeta(relation) {
        const mainAlias = relation.propertyName;
        const joinAlias = relation.junctionEntityMetadata.tableName;
        const joins = [
            {
                target: joinAlias,
                alias: joinAlias,
                conditions: relation.inverseJoinColumns.map(inverseJoinColumn => ({
                    leftHand: `${joinAlias}.${inverseJoinColumn.propertyName}`,
                    rightHand: `${mainAlias}.${inverseJoinColumn.referencedColumn.propertyName}`,
                })),
            },
        ];
        const fromPrimaryKeys = relation.inverseRelation.entityMetadata.primaryColumns.map(pk => ({
            selectPath: `${mainAlias}.${pk.propertyName}`,
            databasePath: pk.databasePath,
            propertyName: pk.propertyName,
        }));
        const entityIds = relation.joinColumns.map(joinColumn => `${joinAlias}.${joinColumn.propertyName}`);
        const entityIdSelects = relation.joinColumns.map(joinColumn => `${joinAlias}.${joinColumn.propertyName} AS ${this.createEntityIdSelect(joinColumn.referencedColumn.propertyPath)}`);
        return {
            from: relation.type,
            fromAlias: mainAlias,
            fromPrimaryKeys,
            entityIds,
            entityIdSelects,
            joins,
            whereCondition(entity, entityIndex) {
                const params = {};
                const sql = relation.joinColumns
                    .map(joinColumn => {
                    const paramName = `${joinColumn.propertyName}_${entityIndex}`;
                    params[paramName] = joinColumn.referencedColumn.getEntityValue(entity);
                    return `${joinAlias}.${joinColumn.propertyName} = :${paramName}`;
                })
                    .join(' AND ');
                return { sql, params };
            },
        };
    }
    getManyToManyNotOwnerMetadata(relation) {
        const mainAlias = relation.propertyName;
        const joinAlias = relation.junctionEntityMetadata.tableName;
        const joins = [
            {
                target: joinAlias,
                alias: joinAlias,
                conditions: relation.inverseRelation.joinColumns.map(joinColumn => ({
                    leftHand: `${joinAlias}.${joinColumn.propertyName}`,
                    rightHand: `${mainAlias}.${joinColumn.referencedColumn.propertyName}`,
                })),
            },
        ];
        const fromPrimaryKeys = relation.inverseRelation.entityMetadata.primaryColumns.map(pk => ({
            selectPath: `${mainAlias}.${pk.propertyName}`,
            databasePath: pk.databasePath,
            propertyName: pk.propertyName,
        }));
        const entityIds = relation.inverseRelation.inverseJoinColumns.map(inverseJoinColumn => `${joinAlias}.${inverseJoinColumn.propertyName}`);
        const entityIdSelects = relation.inverseRelation.inverseJoinColumns.map(inverseJoinColumn => `${joinAlias}.${inverseJoinColumn.propertyName} AS ${this.createEntityIdSelect(inverseJoinColumn.referencedColumn.propertyPath)}`);
        return {
            from: relation.type,
            fromAlias: mainAlias,
            fromPrimaryKeys,
            entityIds,
            entityIdSelects,
            joins,
            whereCondition(entity, entityIndex) {
                const params = {};
                const sql = relation
                    .inverseRelation.inverseJoinColumns.map(inverseJoinColumn => {
                    const paramName = `${inverseJoinColumn.propertyName}_${entityIndex}`;
                    params[paramName] = inverseJoinColumn.referencedColumn.getEntityValue(entity);
                    return `${joinAlias}.${inverseJoinColumn.propertyName} = :${paramName}`;
                })
                    .join(' AND ');
                return { sql, params };
            },
        };
    }
    createEntityIdSelect(propertyPath) {
        return this.escape(`__nestjsQueryEntityId_${propertyPath}__`);
    }
    static isEntityIdCol(colName) {
        return /^__nestjsQueryEntityId_(.*)__$/.test(colName);
    }
    static parseEntityIdFromName(colName) {
        return colName.replace(/^__nestjsQueryEntityId_/, '').replace(/__$/, '');
    }
    getRelationPrimaryKeysPropertyNameAndColumnsName() {
        return this.relationMeta.fromPrimaryKeys.map(pk => ({
            propertyName: pk.propertyName,
            columnName: DriverUtils_1.DriverUtils.buildColumnAlias(this.relationRepo.manager.connection.driver, this.relationMeta.fromAlias, pk.databasePath),
        }));
    }
}
exports.RelationQueryBuilder = RelationQueryBuilder;
//# sourceMappingURL=relation-query.builder.js.map