"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_builder_abstract_1 = require("./query-builder.abstract");
/**
 * @internal
 * Builder to create SQL Comparisons. (=, !=, \>, etc...)
 */
class SQLComparisionBuilder extends query_builder_abstract_1.AbstractQueryBuilder {
    constructor(repo, comparisonMap = SQLComparisionBuilder.DEFAULT_COMPARISON_MAP) {
        super(repo);
        this.comparisonMap = comparisonMap;
        this.paramCount = 0;
    }
    get paramName() {
        const param = `param${this.paramCount}`;
        this.paramCount += 1;
        return param;
    }
    /**
     * Creates a valid SQL fragment with parameters.
     *
     * @param field - the property in Entity to create the comparison for.
     * @param cmp - the FilterComparisonOperator (eq, neq, gt, etc...)
     * @param val - the value to compare to.
     */
    build(field, cmp, val, alias) {
        const col = alias ? `${alias}.${field}` : `${field}`;
        const normalizedCmp = cmp.toLowerCase();
        if (this.comparisonMap[normalizedCmp]) {
            // comparison operator (e.b. =, !=, >, <)
            return this.createComparisonSQL(normalizedCmp, col, val);
        }
        if (normalizedCmp === 'is') {
            // is comparision (IS TRUE, IS FALSE, IS NULL)
            return this.isComparisonSQL(col, val);
        }
        if (normalizedCmp === 'isnot') {
            // is comparision (IS NOT TRUE, IS NOT FALSE, IS NOT NULL, etc...)
            return this.isNotComparisonSQL(col, val);
        }
        if (normalizedCmp === 'in') {
            // in comparision (field IN (1,2,3))
            return this.inComparisonSQL(col, val);
        }
        if (normalizedCmp === 'notin') {
            // in comparision (field IN (1,2,3))
            return this.notInComparisionSQL(col, val);
        }
        throw new Error(`unknown operator "${cmp}"`);
    }
    createComparisonSQL(cmp, col, val) {
        const operator = this.comparisonMap[cmp];
        const { paramName } = this;
        return { sql: `${col} ${operator} :${paramName}`, params: { [paramName]: val } };
    }
    isComparisonSQL(col, val) {
        if (val === null) {
            return { sql: `${col} IS NULL`, params: {} };
        }
        if (val === true) {
            return { sql: `${col} IS TRUE`, params: {} };
        }
        if (val === false) {
            return { sql: `${col} IS FALSE`, params: {} };
        }
        throw new Error(`unexpected is operator param ${val}`);
    }
    isNotComparisonSQL(col, val) {
        if (val === null) {
            return { sql: `${col} IS NOT NULL`, params: {} };
        }
        if (val === true) {
            return { sql: `${col} IS NOT TRUE`, params: {} };
        }
        if (val === false) {
            return { sql: `${col} IS NOT FALSE`, params: {} };
        }
        throw new Error(`unexpected isNot operator param ${val}`);
    }
    inComparisonSQL(col, val) {
        this.checkNonEmptyArray(val);
        const { paramName } = this;
        return {
            sql: `${col} IN (:...${paramName})`,
            params: { [paramName]: val },
        };
    }
    notInComparisionSQL(col, val) {
        this.checkNonEmptyArray(val);
        const { paramName } = this;
        return {
            sql: `${col} NOT IN (:...${paramName})`,
            params: { [paramName]: val },
        };
    }
    checkNonEmptyArray(val) {
        if (!Array.isArray(val)) {
            throw new Error(`Invalid in value expected an array got ${JSON.stringify(val)}`);
        }
        if (!val.length) {
            throw new Error(`Invalid in value expected a non-empty array got ${JSON.stringify(val)}`);
        }
    }
}
exports.SQLComparisionBuilder = SQLComparisionBuilder;
SQLComparisionBuilder.DEFAULT_COMPARISON_MAP = {
    eq: '=',
    neq: '!=',
    gt: '>',
    gte: '>=',
    lt: '<',
    lte: '<=',
    like: 'LIKE',
    notlike: 'NOT LIKE',
    ilike: 'ILIKE',
    notilike: 'NOT ILIKE',
};
//# sourceMappingURL=sql-comparison.builder.js.map