"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeorm_1 = require("typeorm");
const sql_comparison_builder_1 = require("./sql-comparison.builder");
/**
 * @internal
 * Builds a WHERE clause from a Filter.
 */
class WhereBuilder {
    constructor(repository, sqlComparisionBuilder = new sql_comparison_builder_1.SQLComparisionBuilder(repository)) {
        this.repository = repository;
        this.sqlComparisionBuilder = sqlComparisionBuilder;
    }
    /**
     * Builds a WHERE clause from a Filter.
     * @param where - the `typeorm` WhereExpression
     * @param filter - the filter to build the WHERE clause from.
     * @param alias - optional alias to use to qualify an identifier
     */
    build(where, filter, alias) {
        const { and, or } = filter;
        if (and && and.length) {
            this.filterAnd(where, and, alias);
        }
        if (or && or.length) {
            this.filterOr(where, or, alias);
        }
        return this.filterFields(where, filter, alias);
    }
    /**
     * ANDs multiple filters together. This will properly group every clause to ensure proper precedence.
     *
     * @param where - the `typeorm` WhereExpression
     * @param filters - the array of filters to AND together
     * @param alias - optional alias to use to qualify an identifier
     */
    filterAnd(where, filters, alias) {
        return filters.reduce((w, f) => w.andWhere(this.createBrackets(f, alias)), where);
    }
    /**
     * ORs multiple filters together. This will properly group every clause to ensure proper precedence.
     *
     * @param where - the `typeorm` WhereExpression
     * @param filters - the array of filters to OR together
     * @param alias - optional alias to use to qualify an identifier
     */
    filterOr(where, filter, alias) {
        return filter.reduce((w, f) => where.orWhere(this.createBrackets(f, alias)), where);
    }
    /**
     * Wraps a filter in brackes to ensure precedence.
     * ```
     * {a: { eq: 1 } } // "(a = 1)"
     * {a: { eq: 1 }, b: { gt: 2 } } // "((a = 1) AND (b > 2))"
     * ```
     * @param filter - the filter to wrap in brackets.
     * @param alias - optional alias to use to qualify an identifier
     */
    createBrackets(filter, alias) {
        return new typeorm_1.Brackets(qb => this.build(qb, filter, alias));
    }
    /**
     * Creates field comparisons from a filter. This method will ignore and/or properties.
     * @param where - the `typeorm` WhereExpression
     * @param filter - the filter with fields to create comparisons for.
     * @param alias - optional alias to use to qualify an identifier
     */
    filterFields(where, filter, alias) {
        return Object.keys(filter).reduce((w, field) => {
            if (field !== 'and' && field !== 'or') {
                return this.withFilterComparison(where, field, this.getField(filter, field), alias);
            }
            return w;
        }, where);
    }
    getField(obj, field) {
        return obj[field];
    }
    withFilterComparison(where, field, cmp, alias) {
        return where.andWhere(new typeorm_1.Brackets(qb => {
            const opts = Object.keys(cmp);
            const sqlComparisons = opts.map(cmpType => this.sqlComparisionBuilder.build(field, cmpType, cmp[cmpType], alias));
            sqlComparisons.map(({ sql, params }) => qb.orWhere(sql, params));
        }));
    }
}
exports.WhereBuilder = WhereBuilder;
//# sourceMappingURL=where.builder.js.map